#include <Wire.h>
#include <U8g2lib.h>
#include <MPU6050.h>
#include <SPI.h>

#define OLED_DC 8
#define OLED_CS 10
#define OLED_RST 9

U8G2_SH1106_128X64_NONAME_F_4W_HW_SPI u8g2(U8G2_R0, OLED_CS, OLED_DC, OLED_RST);
MPU6050 mpu;

const char answers[][30] PROGMEM = {
  "It is certain",
  "It is decidedly so",
  "Without a doubt",
  "Yes, definitely",
  "You may rely on it",
  "As I see it, yes",
  "Most likely",
  "Outlook good",
  "Yes",
  "Signs point to yes",
  "Reply hazy try again",
  "Ask again later",
  "Better not tell you now",
  "Cannot predict now",
  "Concentrate and ask again",
  "Don't count on it",
  "My reply is no",
  "My sources say no",
  "Outlook not so good",
  "Very doubtful"
};

const int shakeThreshold = 20000;
unsigned long previousMillis = 0;
bool displayAnswer = false;
int num_answers;

void setup() {
  Serial.begin(115200);
  Wire.begin();
  SPI.begin();

  u8g2.begin();
  u8g2.setFont(u8g2_font_ncenB08_tr);

  mpu.initialize();
  if (!mpu.testConnection()) {
    while (1)
      ;
  }

  num_answers = sizeof(answers) / sizeof(answers[0]);
  displayText("Shake me!");
}

void loop() {
  int16_t ax, ay, az;
  mpu.getAcceleration(&ax, &ay, &az);
  if (isShaking(ax, ay, az)) {
    unsigned long currentMillis = millis();
    if (currentMillis - previousMillis >= 500) {
      previousMillis = currentMillis;
      displayAnswer = true;
    }
  }
  if (displayAnswer) {
    displayAnswer = false;
    displayText("Thinking...");
    delay(2000);  // Display "Thinking..." for 2 seconds
    int answer_picked = random(0, num_answers);
    char buffer[30];
    strcpy_P(buffer, answers[answer_picked]);  // Copy the answer from flash memory
    displayText(buffer);
    delay(2000);               // Display the answer for 2 seconds
    displayText("Shake me!");  // Then revert back to "Shake me!"
  }
}

bool isShaking(int16_t ax, int16_t ay, int16_t az) {
  const int16_t deadZone = 8000;
  return (abs(ax) > shakeThreshold && abs(ax) > deadZone) || (abs(ay) > shakeThreshold && abs(ay) > deadZone) || (abs(az) > shakeThreshold && abs(az) > deadZone);
}

void displayText(const char* text) {
  u8g2.clearBuffer();

  // Calculate the width and height of the text
  uint8_t w = u8g2.getStrWidth(text);
  uint8_t h = u8g2.getAscent() - u8g2.getDescent();

  // Calculate positions to center the text
  uint8_t x = (u8g2.getDisplayWidth() - w) / 2;
  uint8_t y = (u8g2.getDisplayHeight() - h) / 2;

  u8g2.drawStr(x, y + u8g2.getAscent(), text);

  u8g2.sendBuffer();
}